@@ -31,7 +31,7 @@
 
 #include <config.h>
 
-#ident "$Id: sub.c 1980 2008-04-27 00:40:09Z nekral-guest $"
+#ident "$Id: sub.c 3232 2010-08-22 19:13:53Z nekral-guest $"
 
 #include <pwd.h>
 #include <stdio.h>
@@ -58,7 +58,7 @@ void subsystem (const struct passwd *pw)
 		printf (_("Invalid root directory '%s'\n"), pw->pw_dir);
 		SYSLOG ((LOG_WARN, BAD_SUBROOT2, pw->pw_dir, pw->pw_name));
 		closelog ();
-		exit (1);
+		exit (EXIT_FAILURE);
 	}
 
 	/*
@@ -66,11 +66,13 @@ void subsystem (const struct passwd *pw)
 	 * must be able to change into it.
 	 */
 
-	if (chdir (pw->pw_dir) || chroot (pw->pw_dir)) {
-		printf (_("Can't change root directory to '%s'\n"),
-			pw->pw_dir);
+	if (   (chdir (pw->pw_dir) != 0)
+	    || (chroot (pw->pw_dir) != 0)) {
+		(void) printf (_("Can't change root directory to '%s'\n"),
+		               pw->pw_dir);
 		SYSLOG ((LOG_WARN, NO_SUBROOT2, pw->pw_dir, pw->pw_name));
 		closelog ();
-		exit (1);
+		exit (EXIT_FAILURE);
 	}
 }
+
