@@ -37,7 +37,7 @@
 
 #include <config.h>
 
-#ident "$Id: grpunconv.c 3640 2011-11-19 21:51:52Z nekral-guest $"
+#ident "$Id: grpunconv.c 3726 2012-05-18 19:32:32Z nekral-guest $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -205,13 +205,7 @@ int main (int argc, char **argv)
 		}
 	}
 
-	if (sgr_close () == 0) {
-		fprintf (stderr,
-		         _("%s: failure while writing changes to %s\n"),
-		         Prog, sgr_dbname ());
-		SYSLOG ((LOG_ERR, "failure while writing changes to %s", sgr_dbname ()));
-		fail_exit (3);
-	}
+	(void) sgr_close (); /* was only open O_RDONLY */
 
 	if (gr_close () == 0) {
 		fprintf (stderr,
